"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
class CheckJobs {
    constructor() {
        this.ec2 = new AWS.EC2();
        this.MINUTE_IN_SECONDS = 60;
        this.HOUR_IN_SECONDS = 60 * this.MINUTE_IN_SECONDS;
        this.DAY_IN_SECONDS = 24 * this.HOUR_IN_SECONDS;
    }
    async getInstances() {
        const filters = [{
                Name: 'tag:LambdaFunction',
                Values: ['nshmp-haz-s3-lambda']
            }];
        const results = await this.ec2.describeInstances({ Filters: filters }).promise();
        return this.processReservations(results.Reservations);
    }
    processReservations(reservations) {
        const responses = [];
        console.log('\n Reservations: ', JSON.stringify(reservations, null, 2));
        const users = this.getUsers(reservations);
        users.forEach(user => {
            const userReservations = reservations.filter(reservation => {
                return reservation.Instances.find(instance => {
                    return this.getTag(instance, 'User').Value === user;
                });
            });
            const jobs = [];
            userReservations.forEach(reservation => {
                reservation.Instances.forEach(instance => {
                    const job = {
                        jobId: this.getTag(instance, 'JobId').Value,
                        triggerFile: this.getTag(instance, 'TriggerFile').Value,
                        state: instance.State.Name,
                        instanceType: instance.InstanceType,
                        launchTime: this.getLaunchTime(instance),
                        s3OutputPath: this.getTag(instance, 'nshmp:s3-output-path').Value,
                        configFile: this.getTag(instance, 'nshmp:config-file').Value,
                        siteFile: this.getTag(instance, 'nshmp:site-file').Value
                    };
                    this.checkRunning(instance, job);
                    jobs.push(job);
                });
            });
            responses.push({
                user: user,
                jobs
            });
        });
        return {
            status: 'success',
            results: responses
        };
    }
    getUsers(reservations) {
        const users = new Set();
        reservations.forEach(reservation => {
            reservation.Instances.forEach(instance => {
                users.add(this.getTag(instance, 'User').Value);
            });
        });
        return users;
    }
    checkRunning(instance, job) {
        if (instance.State.Name == 'running') {
            const now = new Date();
            const diffInSeconds = new Date(now.getTime() - instance.LaunchTime.getTime())
                .getTime() / 1000;
            let runTime = '';
            if (diffInSeconds >= this.DAY_IN_SECONDS) {
                runTime = `${(diffInSeconds / this.DAY_IN_SECONDS).toFixed(4)} days`;
            }
            else if (diffInSeconds >= this.HOUR_IN_SECONDS && diffInSeconds < this.DAY_IN_SECONDS) {
                runTime = `${(diffInSeconds / this.HOUR_IN_SECONDS).toFixed(4)} hours`;
            }
            else {
                runTime = `${(diffInSeconds / this.MINUTE_IN_SECONDS).toFixed(4)} minutes`;
            }
            job.runTime = runTime;
        }
    }
    getLaunchTime(instance) {
        const date = instance.LaunchTime.toLocaleDateString();
        const timeGMT = instance.LaunchTime.toLocaleTimeString();
        const timeMST = instance.LaunchTime
            .toLocaleTimeString('en-US', { timeZone: 'America/Denver' });
        return `${date} ${timeGMT} GMT (${timeMST} MST)`;
    }
    getTag(instance, key) {
        return instance.Tags.find(tag => tag.Key == key);
    }
}
exports.CheckJobs = CheckJobs;
