"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const fs = require("fs");
const Joi = require("joi");
const path = require("path");
const YAML = require("yaml");
const lambda_config_1 = require("./lambda-config");
const nshmp_lambda_util_1 = require("../lambda/nshmp-lambda-util");
/**
 * Get Lambda config.
 */
class NshmpHazLambdaConfig {
    constructor(jobId) {
        this.lambdaConfig = new lambda_config_1.LambdaConfig();
        this.ssm = new AWS.SSM();
        this.s3 = new AWS.S3();
        this.sns = new AWS.SNS();
        this.iam = new AWS.IAM();
        /**
         * Default options for trigger file.
         * Slices at:
         *    - 4.04E-4 (2% in 50 years)
         *    - 1.03E-3 (5% in 50 years)
         *    - 2.10E-3 (10% in 50 years)
         */
        this.defaultOptions = {
            nshmp: {
                slices: [0.000404, 0.00103, 0.0021],
                useCustomModel: false,
                s3Bucket: 'nshmp-hazout',
                s3OutputPath: null
            },
            ec2: {
                accessVisualVM: true
            }
        };
        this.lambdaConfig.jobId = jobId;
    }
    /**
     * Return the Lambda config:
     *  - LambdaTriggerConfig
     *  - Raw YAML trigger file contents
     *  - Job ID
     *  - YAML trigger file name
     *  - NshmpLambdaConfig:
     *      - site file contents
     *      - config file contents
     *      - NSHM
     *      - AWS SNS topic
     *  - Ec2LambdaConfig:
     *      - ec2 image ID
     *      - ec2 instance profile
     *      - ec2 user data contents
     */
    async getConfig(s3Event) {
        const bucketName = s3Event.s3.bucket.name;
        const fileName = s3Event.s3.object.key;
        try {
            const res = await this.s3.getObject({
                Bucket: bucketName,
                Key: fileName
            }).promise();
            this.setTriggerFile(res, fileName);
            await this.checkEmail();
            await this.setSNSSubscriberTopic();
            await this.checkTriggerFile();
            await this.setImageId();
            await this.setIamRole();
            await this.setUserData();
            return this.lambdaConfig;
        }
        catch (error) {
            if (error.isJoi) {
                throw new nshmp_lambda_util_1.LambdaError(`${error.name}: ${error.details[0].message}`, this.lambdaConfig);
            }
            else {
                throw new nshmp_lambda_util_1.LambdaError(error.message, this.lambdaConfig);
            }
        }
    }
    setTriggerFile(response, fileName) {
        const raw = response.Body.toString();
        const triggerConfig = YAML.parse(raw);
        triggerConfig.ec2 = Object.assign({}, this.defaultOptions.ec2, triggerConfig.ec2);
        triggerConfig.nshmp = Object.assign({}, this.defaultOptions.nshmp, triggerConfig.nshmp);
        triggerConfig.nshmp.s3Bucket = this.defaultOptions.nshmp.s3Bucket;
        const user = triggerConfig.nshmp.email.split('@').shift();
        const s3OutputPath = triggerConfig.nshmp.s3OutputPath;
        triggerConfig.nshmp.s3OutputPath = s3OutputPath == null ? user : `${user}/${s3OutputPath}`;
        this.lambdaConfig.rawTriggerConfig = YAML.stringify(triggerConfig);
        this.lambdaConfig.triggerConfig = triggerConfig;
        this.lambdaConfig.triggerFileName = fileName.split('/').pop();
        console.log('\nconfig', JSON.stringify(this.lambdaConfig.triggerConfig, null, 2));
    }
    async checkEmail() {
        const schema = Joi.object().keys({
            nshmp: Joi.object({
                email: Joi.string().email().required()
            }).unknown()
        }).unknown();
        try {
            return await Joi.validate(this.lambdaConfig.triggerConfig, schema);
        }
        catch (error) {
            throw new nshmp_lambda_util_1.LambdaError(error.message, this.lambdaConfig);
        }
    }
    async checkTriggerFile() {
        const triggerConfig = this.lambdaConfig.triggerConfig;
        const schema = Joi.object().keys({
            nshmp: Joi.object({
                email: Joi.string().email().required(),
                nshmpConfigFile: Joi.string().uri().allow(null),
                nshmpModel: Joi.when('useCustomModel', {
                    is: true,
                    then: Joi.string().uri().required(),
                    otherwise: Joi.string().required()
                }),
                nshmpSiteFile: Joi.string().uri().required(),
                program: Joi.string().required(),
                iml: Joi.when(triggerConfig.nshmp.program, {
                    is: 'deagg-iml',
                    then: Joi.number().required(),
                    otherwise: Joi.strip()
                }),
                returnPeriod: Joi.when(triggerConfig.nshmp.program, {
                    is: 'deagg',
                    then: Joi.number().required(),
                    otherwise: Joi.strip()
                }),
                sendUpdates: Joi.boolean().required(),
                sendUpdatesEvery: Joi.when('sendUpdates', {
                    is: true,
                    then: Joi.number().required(),
                    otherwise: Joi.number().allow(null)
                }),
                s3Bucket: Joi.string(),
                s3OutputPath: Joi.string().allow(null, ''),
                s3OutputDirectory: Joi.string().required(),
                slices: Joi.array().allow(null),
                useCustomModel: Joi.boolean().allow(null)
            }).required(),
            ec2: Joi.object({
                instanceType: Joi.string().required(),
                javaXmx: Joi.string().required(),
                accessVisualVM: Joi.boolean()
            }).required()
        });
        try {
            return await Joi.validate(triggerConfig, schema);
        }
        catch (error) {
            throw new nshmp_lambda_util_1.LambdaError(error.message, this.lambdaConfig);
        }
    }
    /**
     * Get the latest EC2 AMI
     *
     * @param config The nshmp trigger config
     */
    async setImageId() {
        const response = await this.ssm.getParameters({
            Names: ['/aws/service/ecs/optimized-ami/amazon-linux-2/recommended/image_id']
        }).promise();
        this.lambdaConfig.ec2.ec2ImageId = response.Parameters[0].Value;
        return;
    }
    /**
     * Get IAM instance profile for ec2.
     *
     * @param config The nshmp trigger config
     */
    async setIamRole() {
        const response = await this.iam.getInstanceProfile({
            InstanceProfileName: 'csr-Ec2-Role'
        }).promise();
        this.lambdaConfig.ec2.ec2InstanceProfile = response.InstanceProfile.Arn;
        return;
    }
    /**
     * Read in the user data script.
     *
     * @param config The trigger config
     */
    setUserData() {
        const filePath = path.join(__dirname, '../lambda-user-data.sh');
        return new Promise((resolve, reject) => {
            fs.readFile(filePath, { encoding: 'UTF-8' }, (error, data) => {
                if (error) {
                    reject(error);
                }
                this.lambdaConfig.ec2.ec2UserDataContents = data.toString();
                resolve();
            });
        });
    }
    async setSNSSubscriberTopic() {
        const email = this.lambdaConfig.triggerConfig.nshmp.email;
        const response = await this.sns.createTopic({
            Name: `nshmp-haz-${email.replace('@', '-').replace('.', '-')}`
        }).promise();
        const subscription = await this.getSNSSubscriber(email, response.TopicArn);
        const topic = {
            TopicArn: subscription.TopicArn
        };
        this.lambdaConfig.nshmp.snsTopic = topic;
        return;
    }
    async getSNSSubscriber(email, topicArn) {
        const subscriptions = await this.getSNSSubscriptions(topicArn);
        return this.snsSubscribe(email, topicArn, subscriptions);
    }
    async getSNSSubscriptions(topicArn) {
        const response = await this.sns.listSubscriptionsByTopic({
            TopicArn: topicArn
        }).promise();
        return response.Subscriptions;
    }
    ;
    snsSubscribe(email, topicArn, subscriptions) {
        const subscriber = subscriptions.find(sub => email === sub.Endpoint);
        if (subscriber) {
            return Promise.resolve(subscriber);
        }
        else {
            return this.sns.subscribe({
                Protocol: 'email',
                Endpoint: email,
                TopicArn: topicArn,
                ReturnSubscriptionArn: true
            }).promise();
        }
    }
}
exports.NshmpHazLambdaConfig = NshmpHazLambdaConfig;
